/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_PRODUCER_QUEUE_H_
#define __GDAM_PRODUCER_QUEUE_H_

typedef struct _GdamProducerQueueClass GdamProducerQueueClass;
typedef struct _GdamProducerQueue GdamProducerQueue;

#include "gdamsongdatabase.h"
#include <gdam/component/gdamproducer.h>

GtkType gdam_producer_queue_get_type();
#define GDAM_TYPE_PRODUCER_QUEUE              (gdam_producer_queue_get_type ())
#define GDAM_PRODUCER_QUEUE(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_PRODUCER_QUEUE, GdamProducerQueue))
#define GDAM_PRODUCER_QUEUE_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_PRODUCER_QUEUE, GdamProducerQueueClass))
#define GDAM_IS_PRODUCER_QUEUE(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_PRODUCER_QUEUE))
#define GDAM_IS_PRODUCER_QUEUE_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_PRODUCER_QUEUE))

/* TODO: freeze, thaw? . */

struct _GdamProducerQueueClass {
	GdamProducerClass 	producer_class;
	void			(*skip_songs)(GdamProducerQueue*	queue,
					      int			songs);
	void			   (*skip_to)(GdamProducerQueue*	queue,
					      int			song);
	void                  (*add_filename)(GdamProducerQueue*    queue,
	                                      const char*           filename);
	void                  (*set_queue_list)
					     (GdamProducerQueue* queue,
	                                      gpointer           songinfo_list);
	void                  (*set_filename)(GdamProducerQueue*      queue,
	                                      const char*           filename);
};

struct _GdamProducerQueue {
	GdamProducer		producer;
	GdamSongDatabase*       song_database;
	GList*			songinfo_list;
	int 			eof_sub_id;
	int 			queued_eof_sub_id;
	GdamSongInfo*		current_song;
	GdamSongInfo*		queued_song;
	int			current_id;
	int			queued_id;
	gboolean		ignore_set_filename;
	char*                   filename;   /* TODO: get rid of this? */
	int			fade_envelope;
	int			buffer_size;
	gboolean		do_repeat;
};

GdamProducerQueue* gdam_producer_queue_new(GdamChannel*	channel);
void gdam_producer_queue_skip_songs(GdamProducerQueue* 	      queue,
				    int 		      num);
void gdam_producer_queue_skip_to   (GdamProducerQueue* 	      queue,
				    int 		      num);
void gdam_producer_queue_print_queue(GdamProducerQueue*	queue);
void gdam_producer_queue_add_filename(GdamProducerQueue* queue,
				      const char*        filename);
void gdam_producer_queue_set_filename(GdamProducerQueue* queue,
				      const char*        filename);
/*  Counts from 1. */
void gdam_producer_queue_delete_range(GdamProducerQueue* queue,
				      guint lower, guint upper);
/*  Counts from 1. */
void gdam_producer_queue_move_range(GdamProducerQueue* queue,
				    guint lower, guint upper, guint dest);
/*  Counts from 1. */
void gdam_producer_queue_print_range(GdamProducerQueue* queue,
				     guint lower, guint upper, guint dest);
void gdam_producer_queue_set_queue_list
				    (GdamProducerQueue* queue,
				     GList*		songinfo_list);
void gdam_producer_queue_search_by_string
				    (GdamProducerQueue* queue,
				     const char*	search_string,
				     int		direction);


#endif
