/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamsynthsource.h"

static int gdam_synth_source_read(GdamSource* source,
                                  GdamSample* samples,
				  int         num_read)
{
	int n;
	gdouble* synth_buf;
	GdamSynthSourceClass* klass;
	GdamSynthSource* synth_source = GDAM_SYNTH_SOURCE(source);
	if (source->ended)
		return 0;
	synth_buf = (gdouble*) alloca(sizeof(gdouble) * num_read);
	klass = GDAM_SYNTH_SOURCE_CLASS(((GtkObject*)source)->klass);
	n = klass->synthesize(synth_source, synth_buf, num_read);
	if (n < num_read) {
		source->ended = 1;
		return num_read;
	}
	return n;
}

static void gdam_synth_source_init(GdamSynthSource* synth_source)
{
}

static void gdam_synth_source_class_init(GdamSynthSourceClass* klass)
{
	GDAM_SOURCE_CLASS(klass)->read = gdam_synth_source_read;
}

GtkType gdam_synth_source_get_type()
{
	static GtkType synth_source_type = 0;
	if (!synth_source_type) {
		static const GtkTypeInfo synth_source_info = {
			"GdamSynthSource",
			sizeof(GdamSynthSource),
			sizeof(GdamSynthSourceClass),
			(GtkClassInitFunc) gdam_synth_source_class_init,
			(GtkObjectInitFunc) gdam_synth_source_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		synth_source_type = gtk_type_unique(GDAM_TYPE_SOURCE,
		                                 &synth_source_info);
	}
	return synth_source_type;
}
